@@ -29,6 +29,9 @@ if get_platform() == "android":
     from .Interfaces import TCPInterface
     from .Interfaces import UDPInterface
     from .Interfaces import I2PInterface
+    from .Interfaces.Android import RNodeInterface
+    from .Interfaces.Android import SerialInterface
+    from .Interfaces.Android import KISSInterface
 else:
     from .Interfaces import *
 
@@ -119,25 +122,28 @@ class Reticulum:
     # Length of truncated hashes in bits.
     TRUNCATED_HASHLENGTH = 128
 
-    HEADER_MINSIZE = 2+1+(TRUNCATED_HASHLENGTH//8)*1
-    HEADER_MAXSIZE = 2+1+(TRUNCATED_HASHLENGTH//8)*2
-    IFAC_MIN_SIZE  = 1
-    IFAC_SALT      = bytes.fromhex("adf54d882c9a9b80771eb4995d702d4a3e733391b2a0f53f416d9f907e55cff8")
+    HEADER_MINSIZE   = 2+1+(TRUNCATED_HASHLENGTH//8)*1
+    HEADER_MAXSIZE   = 2+1+(TRUNCATED_HASHLENGTH//8)*2
+    IFAC_MIN_SIZE    = 1
+    IFAC_SALT        = bytes.fromhex("adf54d882c9a9b80771eb4995d702d4a3e733391b2a0f53f416d9f907e55cff8")
     
-    MDU            = MTU - HEADER_MAXSIZE - IFAC_MIN_SIZE
+    MDU              = MTU - HEADER_MAXSIZE - IFAC_MIN_SIZE
 
-    RESOURCE_CACHE = 24*60*60
-    JOB_INTERVAL   = 15*60
+    RESOURCE_CACHE   = 24*60*60
+    JOB_INTERVAL     = 5*60
+    CLEAN_INTERVAL   = 15*60
+    PERSIST_INTERVAL = 60*60*12
 
-    router         = None
-    config         = None
+    router           = None
+    config           = None
     
     # The default configuration path will be expanded to a directory
     # named ".reticulum" inside the current users home directory
-    configdir    = os.path.expanduser("~")+"/.reticulum"
-    configpath   = ""
-    storagepath  = ""
-    cachepath    = ""
+    userdir          = os.path.expanduser("~")
+    configdir        = None
+    configpath       = ""
+    storagepath      = ""
+    cachepath        = ""
     
     @staticmethod
     def exit_handler():
@@ -161,7 +167,7 @@ class Reticulum:
         RNS.exit()
 
 
-    def __init__(self,configdir=None, loglevel=None):
+    def __init__(self,configdir=None, loglevel=None, logdest=None):
         """
         Initialises and starts a Reticulum instance. This must be
         done before any other operations, and Reticulum will not
@@ -174,6 +180,17 @@ class Reticulum:
 
         if configdir != None:
             Reticulum.configdir = configdir
+        else:
+            if os.path.isdir("/etc/reticulum") and os.path.isfile("/etc/reticulum/config"):
+                Reticulum.configdir = "/etc/reticulum"
+            elif os.path.isdir(Reticulum.userdir+"/.config/reticulum") and os.path.isfile(Reticulum.userdir+"/.config/reticulum/config"):
+                Reticulum.configdir = Reticulum.userdir+"/.config/reticulum"
+            else:
+                Reticulum.configdir = Reticulum.userdir+"/.reticulum"
+
+        if logdest == RNS.LOG_FILE:
+            RNS.logdest = RNS.LOG_FILE
+            RNS.logfile = Reticulum.configdir+"/logfile"
         
         Reticulum.configpath    = Reticulum.configdir+"/config"
         Reticulum.storagepath   = Reticulum.configdir+"/storage"
@@ -206,6 +223,8 @@ class Reticulum:
         self.is_connected_to_shared_instance = False
         self.is_standalone_instance = False
         self.jobs_thread = None
+        self.last_data_persist = time.time()
+        self.last_cache_clean = 0
 
         if not os.path.isdir(Reticulum.storagepath):
             os.makedirs(Reticulum.storagepath)
@@ -230,7 +249,6 @@ class Reticulum:
             RNS.log("Could not load config file, creating default configuration file...")
             self.__create_default_config()
             RNS.log("Default config file created. Make any necessary changes in "+Reticulum.configdir+"/config and restart Reticulum if needed.")
-            import time
             time.sleep(1.5)
 
         self.__apply_config()
@@ -246,7 +264,7 @@ class Reticulum:
         if self.is_shared_instance:
             self.rpc_listener = multiprocessing.connection.Listener(self.rpc_addr, authkey=self.rpc_key)
             thread = threading.Thread(target=self.rpc_loop)
-            thread.setDaemon(True)
+            thread.daemon = True
             thread.start()
 
         atexit.register(Reticulum.exit_handler)
@@ -256,13 +274,20 @@ class Reticulum:
     def __start_jobs(self):
         if self.jobs_thread == None:
             self.jobs_thread = threading.Thread(target=self.__jobs)
-            self.jobs_thread.setDaemon(True)
+            self.jobs_thread.daemon = True
             self.jobs_thread.start()
 
     def __jobs(self):
         while True:
-            # Clean caches
-            self.__clean_caches()
+            now = time.time()
+
+            if now > self.last_cache_clean+Reticulum.CLEAN_INTERVAL:
+                self.__clean_caches()
+                self.last_cache_clean = time.time()
+
+            if now > self.last_data_persist+Reticulum.PERSIST_INTERVAL:
+                self.__persist_data()
+                self.last_data_persist = time.time()
             
             time.sleep(Reticulum.JOB_INTERVAL)
 
@@ -570,6 +595,7 @@ class Reticulum:
                                     if "kiss_framing" in c and c.as_bool("kiss_framing") == True:
                                         kiss_framing = True
                                     i2p_tunneled = c.as_bool("i2p_tunneled") if "i2p_tunneled" in c else False
+                                    tcp_connect_timeout = c.as_int("connect_timeout") if "connect_timeout" in c else None
 
 
                                     interface = TCPInterface.TCPClientInterface(
@@ -578,7 +604,8 @@ class Reticulum:
                                         c["target_host"],
                                         int(c["target_port"]),
                                         kiss_framing = kiss_framing,
-                                        i2p_tunneled = i2p_tunneled
+                                        i2p_tunneled = i2p_tunneled,
+                                        connect_timeout = tcp_connect_timeout,
                                     )
 
                                     if "outgoing" in c and c.as_bool("outgoing") == False:
@@ -604,12 +631,18 @@ class Reticulum:
                                     i2p_peers = c.as_list("peers") if "peers" in c else None
                                     connectable = c.as_bool("connectable") if "connectable" in c else False
 
+                                    if ifac_size == None:
+                                        ifac_size = 16
+
                                     interface = I2PInterface.I2PInterface(
                                         RNS.Transport,
                                         name,
                                         Reticulum.storagepath,
                                         i2p_peers,
                                         connectable = connectable,
+                                        ifac_size = ifac_size,
+                                        ifac_netname = ifac_netname,
+                                        ifac_netkey = ifac_netkey,
                                     )
 
                                     if "outgoing" in c and c.as_bool("outgoing") == False:
@@ -626,10 +659,6 @@ class Reticulum:
                                     interface.announce_cap = announce_cap
                                     if configured_bitrate:
                                         interface.bitrate = configured_bitrate
-                                    if ifac_size != None:
-                                        interface.ifac_size = ifac_size
-                                    else:
-                                        interface.ifac_size = 16
 
                                 if c["type"] == "SerialInterface":
                                     port = c["port"] if "port" in c else None
@@ -887,7 +916,6 @@ class Reticulum:
                 
                 if mode == None:
                     mode = Interface.Interface.MODE_FULL
-                
                 interface.mode = mode
 
                 if configured_bitrate:
@@ -898,7 +926,7 @@ class Reticulum:
                 else:
                     interface.ifac_size = 8
 
-                interface.announce_cap = announce_cap
+                interface.announce_cap = announce_cap if announce_cap != None else Reticulum.ANNOUNCE_CAP/100.0
                 interface.announce_rate_target = announce_rate_target
                 interface.announce_rate_grace = announce_rate_grace
                 interface.announce_rate_penalty = announce_rate_penalty
@@ -928,7 +956,12 @@ class Reticulum:
 
                 RNS.Transport.interfaces.append(interface)
 
+    def _should_persist_data(self):
+        self.__persist_data()
 
+    def __persist_data(self):
+        RNS.Transport.persist_data()
+        RNS.Identity.persist_data()
 
     def __clean_caches(self):
         RNS.log("Cleaning resource and packet caches...", RNS.LOG_EXTREME)
@@ -1040,6 +1073,19 @@ class Reticulum:
                     else:
                         ifstats["i2p_b32"] = None
 
+                if hasattr(interface, "i2p_tunnel_state"):
+                    if interface.i2p_tunnel_state != None:
+                        state_description = "Unknown State"
+                        if interface.i2p_tunnel_state == I2PInterface.I2PInterfacePeer.TUNNEL_STATE_ACTIVE:
+                            state_description = "Tunnel Active"
+                        elif interface.i2p_tunnel_state == I2PInterface.I2PInterfacePeer.TUNNEL_STATE_INIT:
+                            state_description = "Creating Tunnel"
+                        elif interface.i2p_tunnel_state == I2PInterface.I2PInterfacePeer.TUNNEL_STATE_STALE:
+                            state_description = "Tunnel Unresponsive"
+                        ifstats["tunnelstate"] = state_description
+                    else:
+                        ifstats["tunnelstate"] = None
+
                 if hasattr(interface, "bitrate"):
                     if interface.bitrate != None:
                         ifstats["bitrate"] = interface.bitrate
